@@ -15,6 +15,8 @@ from urllib import parse
 from urllib.parse import unquote, urljoin
 
 from pipenv.utils.constants import VCS_LIST
+from pipenv.utils.dependencies import extract_vcs_url
+from pipenv.vendor.tomlkit.items import SingleKey, Table
 
 try:
     import tomllib as toml
@@ -32,7 +34,6 @@ from pipenv.patched.pip._internal.exceptions import ConfigurationError
 from pipenv.patched.pip._internal.models.link import Link
 from pipenv.patched.pip._internal.req.req_install import InstallRequirement
 from pipenv.patched.pip._internal.utils.hashes import FAVORITE_HASH
-from pipenv.patched.pip._vendor import pkg_resources
 from pipenv.utils import err
 from pipenv.utils.constants import is_type_checking
 from pipenv.utils.dependencies import (
@@ -63,13 +64,12 @@ from pipenv.utils.shell import (
     find_windows_executable,
     get_workon_home,
     is_virtual_environment,
-    load_path,
     looks_like_dir,
     safe_expandvars,
     system_which,
 )
 from pipenv.utils.toml import cleanup_toml, convert_toml_outline_tables
-from pipenv.vendor import click, plette, tomlkit
+from pipenv.vendor import plette, tomlkit
 
 try:
     # this is only in Python3.8 and later
@@ -78,6 +78,11 @@ except ImportError:
     # eventually distlib will remove cached property when they drop Python3.7
     from pipenv.patched.pip._vendor.distlib.util import cached_property
 
+if sys.version_info < (3, 10):
+    from pipenv.vendor import importlib_metadata
+else:
+    import importlib.metadata as importlib_metadata
+
 if is_type_checking():
     from typing import Dict, List, Union
 
@@ -129,11 +134,6 @@ def preferred_newlines(f):
     return DEFAULT_NEWLINES
 
 
-# (path, file contents) => TOMLFile
-# keeps track of pipfiles that we've seen so we do not need to re-parse 'em
-_pipfile_cache = {}
-
-
 class SourceNotFound(KeyError):
     pass
 
@@ -236,6 +236,8 @@ class Project:
             return ["packages", "dev-packages"] + list(package_categories)
 
     def get_requests_session_for_source(self, source):
+        if not (source and source.get("name")):
+            return None
         if self.sessions.get(source["name"]):
             session = self.sessions[source["name"]]
         else:
@@ -268,6 +270,8 @@ class Project:
     def get_hashes_from_pypi(self, ireq, source):
         pkg_url = f"https://pypi.org/pypi/{ireq.name}/json"
         session = self.get_requests_session_for_source(source)
+        if not session:
+            return None
         try:
             collected_hashes = set()
             # Grab the hashes from the new warehouse API.
@@ -353,11 +357,8 @@ class Project:
 
         except Exception:
             if self.s.is_verbose():
-                click.echo(
-                    "{}: Error generating hash for {}".format(
-                        click.style("Warning", bold=True, fg="red"), ireq.name
-                    ),
-                    err=True,
+                err.print(
+                    f"[bold red]Warning[/bold red]: Error generating hash for {ireq.name}"
                 )
             return None
 
@@ -448,18 +449,13 @@ class Project:
             return path.absolute().as_posix()
         return str(get_workon_home().joinpath(name))
 
-    @property
-    def working_set(self) -> pkg_resources.WorkingSet:
-        sys_path = load_path(self.which("python"))
-        return pkg_resources.WorkingSet(sys_path)
-
     @property
     def installed_packages(self):
         return self.environment.get_installed_packages()
 
     @property
-    def installed_package_names(self) -> list[str]:
-        return get_canonical_names([pkg.key for pkg in self.installed_packages])
+    def installed_package_names(self):
+        return get_canonical_names([pkg.name for pkg in self.installed_packages])
 
     @property
     def lockfile_package_names(self) -> dict[str, set[str]]:
@@ -512,7 +508,7 @@ class Project:
             self._environment = self.get_environment(allow_global=allow_global)
         return self._environment
 
-    def get_outdated_packages(self) -> list[pkg_resources.Distribution]:
+    def get_outdated_packages(self) -> list[importlib_metadata.Distribution]:
         return self.environment.get_outdated_packages(pre=self.pipfile.get("pre", False))
 
     @classmethod
@@ -669,16 +665,9 @@ class Project:
 
     @property
     def parsed_pipfile(self) -> tomlkit.toml_document.TOMLDocument | TPipfile:
-        """Parse Pipfile into a TOMLFile and cache it
-
-        (call clear_pipfile_cache() afterwards if mutating)"""
+        """Parse Pipfile into a TOMLFile"""
         contents = self.read_pipfile()
-        # use full contents to get around str/bytes 2/3 issues
-        cache_key = (self.pipfile_location, contents)
-        if cache_key not in _pipfile_cache:
-            parsed = self._parse_pipfile(contents)
-            _pipfile_cache[cache_key] = parsed
-        return _pipfile_cache[cache_key]
+        return self._parse_pipfile(contents)
 
     def read_pipfile(self) -> str:
         # Open the pipfile, read it into memory.
@@ -690,10 +679,6 @@ class Project:
 
         return contents
 
-    def clear_pipfile_cache(self) -> None:
-        """Clear pipfile cache (e.g., so we can mutate parsed pipfile)"""
-        _pipfile_cache.clear()
-
     def _parse_pipfile(
         self, contents: str
     ) -> tomlkit.toml_document.TOMLDocument | TPipfile:
@@ -779,12 +764,7 @@ class Project:
         for category in categories:
             lock_section = lockfile.get(category)
             if lock_section is None:
-                lockfile[category] = lock_section = {}
-            for key in list(lock_section.keys()):
-                norm_key = pep423_name(key)
-                specifier = lock_section[key]
-                del lock_section[key]
-                lockfile[category][norm_key] = specifier
+                lockfile[category] = {}
 
         return lockfile
 
@@ -990,8 +970,6 @@ class Project:
         formatted_data = cleanup_toml(formatted_data)
         with open(path, "w", newline=newlines) as f:
             f.write(formatted_data)
-        # pipfile is mutated!
-        self.clear_pipfile_cache()
 
     def write_lockfile(self, content):
         """Write out the lockfile."""
@@ -1023,13 +1001,18 @@ class Project:
         return sources
 
     def get_default_index(self):
-        return self.pipfile_sources()[0]
+        return self.populate_source(self.pipfile_sources()[0])
 
     def get_index_by_name(self, index_name):
         for source in self.pipfile_sources():
             if source.get("name") == index_name:
                 return source
 
+    def get_index_by_url(self, index_url):
+        for source in self.pipfile_sources():
+            if source.get("url") == index_url:
+                return source
+
     @property
     def sources(self):
         if self.lockfile_exists and hasattr(self.lockfile_content, "keys"):
@@ -1087,7 +1070,6 @@ class Project:
 
         sources = (self.sources, self.pipfile_sources())
         if refresh:
-            self.clear_pipfile_cache()
             sources = reversed(sources)
         found = next(
             iter(find_source(source, name=name, url=url) for source in sources), None
@@ -1098,22 +1080,34 @@ class Project:
         return found
 
     def get_package_name_in_pipfile(self, package_name, category):
-        """Get the equivalent package name in pipfile"""
-        section = self.parsed_pipfile.get(category)
-        if section is None:
-            section = {}
-        package_name = pep423_name(package_name)
+        section = self.parsed_pipfile.get(category, {})
+        normalized_name = pep423_name(package_name)
         for name in section:
-            if pep423_name(name) == package_name:
+            if pep423_name(name) == normalized_name:
                 return name
-        return None
+        return package_name  # Return original name if not found
 
-    def _sort_category(self, category):
-        # toml tables won't maintain sorted dictionary order
-        # so construct the table in the order that we need
+    def get_pipfile_entry(self, package_name, category):
+        name = self.get_package_name_in_pipfile(package_name, category)
+        return self.parsed_pipfile.get(category, {}).get(name)
+
+    def _sort_category(self, category) -> Table:
+        # copy table or create table from dict-like object
         table = tomlkit.table()
-        for key, value in sorted(category.items()):
-            table.add(key, value)
+        if isinstance(category, Table):
+            table.update(category.value)
+        else:
+            table.update(category)
+
+        # sort the table internally
+        table._value._body.sort(key=lambda t: t[0] and t[0].key or "")
+        for index, (key, _) in enumerate(table._value._body):
+            assert isinstance(key, SingleKey)
+            indices = table._value._map[key]
+            if isinstance(indices, tuple):
+                table._value._map[key] = (index,) + indices[1:]
+            else:
+                table._value._map[key] = index
 
         return table
 
@@ -1129,6 +1123,16 @@ class Project:
             return True
         return False
 
+    def reset_category_in_pipfile(self, category):
+        # Read and append Pipfile.
+        p = self.parsed_pipfile
+        if category:
+            del p[category]
+            p[category] = {}
+            self.write_toml(p)
+            return True
+        return False
+
     def remove_packages_from_pipfile(self, packages):
         parsed = self.parsed_pipfile
         packages = {pep423_name(pkg) for pkg in packages}
@@ -1141,7 +1145,9 @@ class Project:
                 del parsed[category][pkg_name]
         self.write_toml(parsed)
 
-    def generate_package_pipfile_entry(self, package, pip_line, category=None):
+    def generate_package_pipfile_entry(
+        self, package, pip_line, category=None, index_name=None
+    ):
         """Generate a package entry from pip install line
         given the installreq package and the pip line that generated it.
         """
@@ -1189,7 +1195,9 @@ class Project:
                     vcs_parts = vcs_part.rsplit("@", 1)
                     if len(vcs_parts) > 1:
                         entry["ref"] = vcs_parts[1].split("#", 1)[0].strip()
-                    entry[vcs] = vcs_parts[0].strip()
+                    vcs_url = vcs_parts[0].strip()
+                    vcs_url = extract_vcs_url(vcs_url)
+                    entry[vcs] = vcs_url
 
                     # Check and extract subdirectory fragment
                     if package.link.subdirectory_fragment:
@@ -1197,7 +1205,10 @@ class Project:
                     break
         else:
             entry["version"] = specifier
-        if hasattr(package, "index"):
+
+        if index_name:
+            entry["index"] = index_name
+        elif hasattr(package, "index"):
             entry["index"] = package.index
 
         if len(entry) == 1 and "version" in entry:
@@ -1220,26 +1231,28 @@ class Project:
         newly_added = False
 
         # Read and append Pipfile.
-        p = self.parsed_pipfile
+        parsed_pipfile = self.parsed_pipfile
 
         # Set empty group if it doesn't exist yet.
-        if category not in p:
-            p[category] = {}
+        if category not in parsed_pipfile:
+            parsed_pipfile[category] = {}
 
-        if name and name != normalized_name:
-            self.remove_package_from_pipfile(name, category=category)
+        section = parsed_pipfile.get(category, {})
+        for entry_name in section.copy().keys():
+            if entry_name.lower() == normalized_name.lower():
+                del parsed_pipfile[category][entry_name]
 
         # Add the package to the group.
-        if normalized_name not in p[category]:
+        if normalized_name not in parsed_pipfile[category]:
             newly_added = True
 
-        p[category][normalized_name] = entry
+        parsed_pipfile[category][normalized_name] = entry
 
         if self.settings.get("sort_pipfile"):
-            p[category] = self._sort_category(p[category])
+            parsed_pipfile[category] = self._sort_category(parsed_pipfile[category])
 
         # Write Pipfile.
-        self.write_toml(p)
+        self.write_toml(parsed_pipfile)
         return newly_added, category, normalized_name
 
     def src_name_from_url(self, index_url):
@@ -1260,26 +1273,45 @@ class Project:
         return name
 
     def add_index_to_pipfile(self, index, verify_ssl=True):
-        """Adds a given index to the Pipfile."""
+        """
+        Adds a given index to the Pipfile if it doesn't already exist.
+        Returns the source name regardless of whether it was newly added or already existed.
+        """
         # Read and append Pipfile.
         p = self.parsed_pipfile
         source = None
+
+        # Try to find existing source by URL or name
         try:
             source = self.get_source(url=index)
         except SourceNotFound:
             with contextlib.suppress(SourceNotFound):
                 source = self.get_source(name=index)
 
+        # If we found an existing source with a name, return it
         if source is not None and source.get("name"):
             return source["name"]
-        source = {"url": index, "verify_ssl": verify_ssl}
-        source["name"] = self.src_name_from_url(index)
-        # Add the package to the group.
+
+        # Check if the URL already exists in any source
+        if "source" in p:
+            for existing_source in p["source"]:
+                if existing_source.get("url") == index:
+                    return existing_source.get("name")
+
+        # If we reach here, the source doesn't exist, so create and add it
+        source = {
+            "url": index,
+            "verify_ssl": verify_ssl,
+            "name": self.src_name_from_url(index),
+        }
+
+        # Add the source to the group
         if "source" not in p:
             p["source"] = [tomlkit.item(source)]
         else:
             p["source"].append(tomlkit.item(source))
-        # Write Pipfile.
+
+        # Write Pipfile
         self.write_toml(p)
         return source["name"]
 
@@ -1294,11 +1326,8 @@ class Project:
                 j = json.load(lock)
                 self._lockfile_newlines = preferred_newlines(lock)
             except JSONDecodeError:
-                click.secho(
-                    "Pipfile.lock is corrupted; ignoring contents.",
-                    fg="yellow",
-                    bold=True,
-                    err=True,
+                err.print(
+                    "[bold yellow]Pipfile.lock is corrupted; ignoring contents.[/bold yellow]"
                 )
                 j = {}
         if not j.get("_meta"):
