@@ -12,8 +12,10 @@ def do_lock(
     clear=False,
     pre=False,
     write=True,
+    quiet=False,
     pypi_mirror=None,
     categories=None,
+    extra_pip_args=None,
 ):
     """Executes the freeze functionality."""
     if not pre:
@@ -31,10 +33,10 @@ def do_lock(
             lockfile_categories.insert(0, "default")
     # Create the lockfile.
     lockfile = project.lockfile(categories=lockfile_categories)
-    # for category in lockfile_categories:
-    #     for k, v in lockfile.get(category, {}).copy().items():
-    #         if not hasattr(v, "keys"):
-    #             del lockfile[category][k]
+    for category in lockfile_categories:
+        for k, v in lockfile.get(category, {}).copy().items():
+            if not hasattr(v, "keys"):
+                del lockfile[category][k]
 
     # Resolve package to generate constraints before resolving other categories
     for category in lockfile_categories:
@@ -45,15 +47,15 @@ def do_lock(
             packages = project.get_pipfile_section(pipfile_category)
 
         if write:
-            # Alert the user of progress.
-            click.echo(
-                "{} {} {}".format(
-                    click.style("Locking"),
-                    click.style(f"[{pipfile_category}]", fg="yellow"),
-                    click.style("dependencies..."),
-                ),
-                err=True,
-            )
+            if not quiet:  # Alert the user of progress.
+                click.echo(
+                    "{} {} {}".format(
+                        click.style("Locking"),
+                        click.style(f"[{pipfile_category}]", fg="yellow"),
+                        click.style("dependencies..."),
+                    ),
+                    err=True,
+                )
 
         # Prune old lockfile category as new one will be created.
         with contextlib.suppress(KeyError):
@@ -66,7 +68,7 @@ def do_lock(
             packages,
             which=project._which,
             project=project,
-            category=pipfile_category,
+            pipfile_category=pipfile_category,
             clear=clear,
             pre=pre,
             allow_global=system,
@@ -74,6 +76,7 @@ def do_lock(
             pipfile=packages,
             lockfile=lockfile,
             old_lock_data=old_lock_data,
+            extra_pip_args=extra_pip_args,
         )
 
     # Overwrite any category packages with default packages.
@@ -87,15 +90,16 @@ def do_lock(
     if write:
         lockfile.update({"_meta": project.get_lockfile_meta()})
         project.write_lockfile(lockfile)
-        click.echo(
-            "{}".format(
-                click.style(
-                    f"Updated Pipfile.lock ({project.get_lockfile_hash()})!",
-                    bold=True,
-                )
-            ),
-            err=True,
-        )
+        if not quiet:
+            click.echo(
+                "{}".format(
+                    click.style(
+                        f"Updated Pipfile.lock ({project.get_lockfile_hash()})!",
+                        bold=True,
+                    )
+                ),
+                err=True,
+            )
     else:
         return lockfile
 
